/*
** Module   : SpawnerImpl.java
** Abstract : spawn the web client.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------Description--------------------------
** 001 MAG 20131118 First version.
*/

package com.goldencode.p2j.main;

import java.util.*;
import java.util.concurrent.*;
import java.util.logging.*;

import com.goldencode.p2j.net.*;
import com.goldencode.p2j.util.*;

/**
 * The spawner implementation.
 * 
 */
public class SpawnerImpl
implements Spawner
{
   /** Logger */
   @SuppressWarnings("unused")
   private static final Logger LOG = LogHelper.getLogger(SpawnerImpl.class.getName());

   /** Instance of Spawner */
   private static Spawner spawner;
   
   /** Registered server listeners */
   private Map<String, SpawnerListener> listeners = 
            new ConcurrentHashMap<String, SpawnerListener>();
   
   static
   {
      spawner = new SpawnerImpl();               
   }
   
   /**
    * Get the Spawner instance.
    * 
    * @return  The instance of spawner.
    */
   public static Spawner getInstance()
   {
      return spawner;
   }
   
   /**
    * Register a Remote Object Server.
    */
   private SpawnerImpl()
   {      
      RemoteObject.registerNetworkServer(Spawner.class, this);
   }
      
   /**
    * Notify when the remote embedded server is ready and listen.
    * 
    * @param   uuid
    *          Remote client assigned identifier.
    * @param   uri
    *          The server URI on which the remote server listen.
    */   
   @Override
   public void onRemoteServerListen(String uuid, String uri)
   {
      SpawnerListener listener = listeners.get(uuid);
      listener.onRemoteServerListen(uri);
   }

   /**
    * Export server key store
    * 
    * @param   uuid
    *          Remote client assigned identifier.
    */   
   @Override
   public ServerKeyStore exportServerKeyStore(String uuid)
   {
      SpawnerListener listener = listeners.get(uuid);
      return listener.getServerKeyStore();
   }
   
   /**
    * Add listener.
    * 
    * @param   uuid
    *          Remote client assigned identifier.
    * @param   listener
    *          Listener instance to add.
    */
   public void addListener(String uuid, SpawnerListener listener)
   {
      listeners.put(uuid, listener);
   }

   /**
    * Remove listener.
    * 
    * @param   uuid
    *          Remote client assigned identifier.
    */
   public void removeListener(String uuid)
   {
         listeners.remove(uuid);
   }
}
