/*
** Module   : WebClientSpawner.java
** Abstract : define the Web Client Spawner.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------Description--------------------------
** 001 MAG 20131119 First version.
*/

package com.goldencode.p2j.main;

import java.util.logging.*;

import com.goldencode.p2j.util.*;

/**
 * Class used to lunch a new WebClient process.
 * 
 * @author mag
 *
 */
public class WebClientSpawner
implements SpawnerListener
{
   /** Logger. */
   private static final Logger LOG = LogHelper.getLogger(WebClientSpawner.class.getName());
   
   /** Timeout */
   private static final int TIME_OUT = 5000;
   
   /** Remote server URI */
   private volatile String remoteUri = null;
   
   /** Client Builder */
   private ClientBuilder cb;
   
   /**
    * Lunch a Web Server Client
    * 
    * @param   userName
    *          User name.
    * @param   password
    *          User password. 
    * 
    * @return  The remote server URI or <code>null</code>
    */
   public String spawn(String userName, String password)
   {
      // Client builder
      ClientBuilderParameters cfg = new ClientBuilderParameters();
      cfg.setUserName(userName);
      cfg.setPassword(password);

      cb = new ClientBuilder(cfg);
      // register listener
      SpawnerImpl spawner = (SpawnerImpl) SpawnerImpl.getInstance();
      spawner.addListener(cfg.getUuid(), this);
      
      // spawn     
      Process shell = null;
      try
      {
         shell = cb.start();
         // wait for notification
         synchronized (this)
         {
            this.wait(TIME_OUT);
         }
      }
      catch (Exception e)
      {
         LOG.logp(Level.SEVERE, 
                  "WebHandler.spawn()", "",
                  LogHelper.generate("Cannot spawn the process " + e.getMessage()));
      }
      finally
      {
         spawner.removeListener(cfg.getUuid());
      }
      
      if (remoteUri == null)
      {
         shell.destroy();
      }
      
      return remoteUri;
   }
   
   /**
    * Notification method on remote server running.
    * 
    * @param   uri
    *          Remote server URI.
    */
   @Override
   public void onRemoteServerListen(String uri)
   {
      remoteUri = uri;
      synchronized (this)
      {
         this.notify();
      }
   }

   /**
    * Export server key store
    */
   @Override
   public ServerKeyStore getServerKeyStore()
   {
      return cb.getServerKeyStore();
   }   
}
